import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
  symbolStar,
  scaleOrdinal
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let symbolValue;
  let margin;
  let radius;
 // instance of the chart
  const my = (selection) => {
    const x = scaleLinear()     // x scale function
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()    // y scale function
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const symbolScale = scaleOrdinal()  // symbol type
    .domain(data.map(symbolValue))
    .range(symbols);
// Define Variable SymbolGenerator to generate the symbols    
    const symbolGenerator =  symbol()
    
    const marks = data.map((d) => ({ //variable to instansiate marks to by shown by a random symbol
      x: x(xValue(d)),
      y: y(yValue(d)),
      pathD: symbolGenerator.type(symbolScale(symbolValue(d)))(),
    }));

    const circles = selection
      .selectAll('path')
      .data(marks)
      .join('path')
    	.attr('d',d => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`);
    	;

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
    
    
  };

  // accessors: they work as setters and getters
  
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };
  
   my.symbolValue = function (_) {
    return arguments.length ? ((symbolValue = _), my) : symbolValue;
  };


  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
